useDynLib(clifford, .registration=TRUE)
exportPattern("^[[:alpha:]]+")

export("%.%")
export("%dot%")
export("%^%")
export("%X%")
export("%star%")
export("% %")
export("%euc%")
export("%o%")
export("%_|%")
export("%|_%")


importFrom("Rcpp", "evalCpp")
importFrom("utils", "capture.output")
importFrom("stats", "rnorm")
importFrom("partitions", "allbinom")

importFrom("disordR","disord")
importFrom("disordR","elements")
importFrom("disordR","hashcal")
importFrom("disordR","is.disord")
importFrom("disordR","consistent")
importFrom("disordR","%in%")
importFrom("disordR","lapply")
importFrom("disordR","sort")

importFrom("magrittr", "%<>%")

importFrom("methods","setOldClass")

importClassesFrom("freealg", "dot")
importMethodsFrom("freealg", "[")

S3method("[", clifford)
S3method("[<-", clifford)
S3method("%.%",clifford)
S3method("%^%",clifford)
S3method("%X%",clifford)
S3method("%star%",clifford)
S3method("% %",clifford)
S3method("%euc%",clifford)
S3method("%o%",clifford)
S3method("%_|%",clifford)
S3method("%|_%",clifford)
S3method(Ops,clifford)
S3method(print,clifford)
S3method(rev,clifford)
S3method(Conj,clifford)
S3method(Mod,clifford)
S3method(dim,clifford)
S3method("const<-", clifford)
S3method("coeffs<-", clifford)
S3method("as.character", clifford)
S3method("print",summary.clifford)
S3method("summary",clifford)
S3method("as.vector",clifford)
S3method("Re",clifford)
S3method("Im",clifford)
S3method("is.zero",clifford)

S3method(print,sigobj)
