# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

c_identity <- function(L, p, m) {
    .Call(`_clifford_c_identity`, L, p, m)
}

c_add <- function(L1, c1, L2, c2, m) {
    .Call(`_clifford_c_add`, L1, c1, L2, c2, m)
}

c_multiply <- function(L1, c1, L2, c2, m, sig) {
    .Call(`_clifford_c_multiply`, L1, c1, L2, c2, m, sig)
}

c_power <- function(L, c, m, p, sig) {
    .Call(`_clifford_c_power`, L, c, m, p, sig)
}

c_grade <- function(L, c, m, n) {
    .Call(`_clifford_c_grade`, L, c, m, n)
}

c_equal <- function(L1, c1, L2, c2, m) {
    .Call(`_clifford_c_equal`, L1, c1, L2, c2, m)
}

c_getcoeffs <- function(L, c, m, B) {
    .Call(`_clifford_c_getcoeffs`, L, c, m, B)
}

c_outerprod <- function(L1, c1, L2, c2, m, sig) {
    .Call(`_clifford_c_outerprod`, L1, c1, L2, c2, m, sig)
}

c_innerprod <- function(L1, c1, L2, c2, m, sig) {
    .Call(`_clifford_c_innerprod`, L1, c1, L2, c2, m, sig)
}

c_fatdotprod <- function(L1, c1, L2, c2, m, sig) {
    .Call(`_clifford_c_fatdotprod`, L1, c1, L2, c2, m, sig)
}

c_lefttickprod <- function(L1, c1, L2, c2, m, sig) {
    .Call(`_clifford_c_lefttickprod`, L1, c1, L2, c2, m, sig)
}

c_righttickprod <- function(L1, c1, L2, c2, m, sig) {
    .Call(`_clifford_c_righttickprod`, L1, c1, L2, c2, m, sig)
}

c_overwrite <- function(L1, c1, L2, c2, m) {
    .Call(`_clifford_c_overwrite`, L1, c1, L2, c2, m)
}

c_cartan <- function(L, c, m, n) {
    .Call(`_clifford_c_cartan`, L, c, m, n)
}

c_cartan_inverse <- function(L, c, m, n) {
    .Call(`_clifford_c_cartan_inverse`, L, c, m, n)
}

